library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity la_blockram is
	generic (
					WIDTH:positive:=8;
					DEPTH:positive:=2048
	);
	port (
		clk: in std_logic;
		reset: in std_logic;
		trigger: in std_logic;
		read_en: in std_logic;
		data_out: in std_logic;
		read_finish:out std_logic;
		reset_cnt : in std_logic;
	 	trigger_address: out std_logic_vector(10 downto 0);
		aqui_finished:out std_logic;
		din: in std_logic_vector(WIDTH-1 downto 0);
		dout: out std_logic_vector(WIDTH-1 downto 0)
	);
end la_blockram;

architecture rtl of la_blockram is

-- infer RAM

type ram_type is array (DEPTH-1 downto 0) of std_logic_vector (WIDTH-1 downto 0);
signal cycle_ram : ram_type; 

-- 
type aquisition is (IDLE,AQUI,AQUI_FIN);
signal astate,astate_next : aquisition;

-- signals

signal writeaddress,writeaddress_next: std_logic_vector(10 downto 0); -- 2^11=2048 Byte;
signal readaddress,readaddress_next: std_logic_vector(10 downto 0); -- 2^11=2048 Byte;
signal aquicounter,aquicounter_next: std_logic_vector(10 downto 0); -- 2^11=2048 Byte;
signal aquicounter_en,aquicounter_reset:std_logic;

signal write_en: std_logic:='1';



constant RESET_ACTIVE: std_logic:='1';

begin

write_en<='1';


-- counter

write_address_counter_seq: process(clk,reset)
begin
	if reset=RESET_ACTIVE then
		writeaddress<=(others=>'0');
	elsif rising_edge(clk) then
			writeaddress<=writeaddress_next;
	end if;
end process;

write_address_counter_com:process(writeaddress,write_en)
begin
	-- default statment
	writeaddress_next<=writeaddress;
	if (write_en='1') then
		writeaddress_next<=unsigned(writeaddress)+1;
	end if;
	if (writeaddress=DEPTH-1) then
		writeaddress_next<=(others=>'0');
	end if;
end process;

read_address_counter_seq: process(clk,reset)
begin
	if reset=RESET_ACTIVE then
		readaddress<=(others=>'0');
	elsif rising_edge(clk) then
		readaddress<=readaddress_next;
	end if;
end process;

read_address_counter_com:process(readaddress,reset_cnt,read_en)
begin
	--default statement
	readaddress_next<=readaddress;

	if (read_en='1') then
		readaddress_next<=unsigned(readaddress)+1;
	end if;
	if (reset_cnt='1') then
		readaddress_next<=(others=>'0');
	end if;
end process;

read_finish<='1' when (readaddress=DEPTH-1) else '0';

-- ram access

write_data: process(clk,reset)
begin
	if rising_edge(clk) then
		if (write_en='1' and aquicounter_en='1') then
			cycle_ram(conv_integer(unsigned(writeaddress)))<= din;
		end if;
	end if;
end process;

read_data: process(clk)
begin
	if rising_edge(clk) then
		if (data_out='1') then
			dout<=cycle_ram(conv_integer(unsigned(readaddress)));
		end if;
	end if;
end process;

-- save trigger point

save_trigger: process(clk,reset)
begin
	if (reset=RESET_ACTIVE) then
		trigger_address<=(others=>'0');
	elsif rising_edge(clk) then
		if (trigger='1') then
			trigger_address<=writeaddress;
		end if;
	end if;	
end process;

-- aqui cnt
aqui_cnt_seq:process(reset,clk)
begin
	if (reset=RESET_ACTIVE) then
		aquicounter<=(others=>'0');
	elsif rising_edge(clk) then
		aquicounter<=aquicounter_next;
	end if;
end process;

aqui_cnt_com:process(aquicounter,aquicounter_en,aquicounter_reset)
begin
	--default statement
	aquicounter_next<=aquicounter;
	if (aquicounter_en='1') then
		aquicounter_next<=aquicounter+1;
	end if;
	if (aquicounter_reset='1') then
		aquicounter_next<=(others=>'0');
	end if;
end process;
 
-- aqui statemachine
aqui_seq:process(reset,clk)
begin
	if (reset=RESET_ACTIVE) then
		astate<=IDLE;
	elsif rising_edge(clk) then
		astate<=astate_next;
	end if;
end process;


aqui_com:process(astate,trigger,aquicounter)
begin
	-- default statement
	astate_next<=astate;
	aqui_finished<='0';
	aquicounter_en<='0';
	aquicounter_reset<='0';

	case astate is
		when IDLE =>	aquicounter_reset<='1';	 
							if (trigger='1') then
								astate_next<=AQUI;
							end if;
		when AQUI =>	aquicounter_en<='1';
							if (aquicounter=DEPTH-1) then
								astate_next<=AQUI_FIN;
							end if;
		when AQUI_FIN => 
							aqui_finished<='1';
							astate_next<=IDLE;								
	end case;
end process;

end rtl;
